@@ -85,6 +85,21 @@ static struct {
 	DB_ENV *env;
 } global_env;
 
+static EBookBackendSyncStatus
+db_error_to_status (const int db_error)
+{
+	switch (db_error) {
+	case 0:
+		return GNOME_Evolution_Addressbook_Success;
+	case DB_NOTFOUND:
+		return GNOME_Evolution_Addressbook_ContactNotFound;
+	case EACCES:
+		return GNOME_Evolution_Addressbook_PermissionDenied;
+	default:
+		return GNOME_Evolution_Addressbook_OtherError;
+	}
+}
+
 static void
 string_to_dbt(const char *str, DBT *dbt)
 {
@@ -115,7 +130,7 @@ build_summary (EBookBackendFilePrivate *bfpriv)
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
+		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 		return;
 	}
 
@@ -165,29 +180,33 @@ set_revision (EContact *contact)
 	
 }
 
-static EContact *
+static EBookBackendSyncStatus
 do_create(EBookBackendFile  *bf,
-	  const char      *vcard_req)
+	  const char      *vcard_req,
+	  EContact **contact)
 {
 	DB             *db = bf->priv->file_db;
 	DBT            id_dbt, vcard_dbt;
 	int            db_error;
 	char           *id;
-	EContact       *contact;
 	char           *vcard;
 	const char *rev;
 	
+	g_assert (bf);
+	g_assert (vcard_req);
+	g_assert (contact);
+
 	id = e_book_backend_file_create_unique_id ();
 
 	string_to_dbt (id, &id_dbt);
 
-	contact = e_contact_new_from_vcard (vcard_req);
-	e_contact_set (contact, E_CONTACT_UID, id);
-	rev = e_contact_get_const (contact,  E_CONTACT_REV);
+	*contact = e_contact_new_from_vcard (vcard_req);
+	e_contact_set (*contact, E_CONTACT_UID, id);
+	rev = e_contact_get_const (*contact,  E_CONTACT_REV);
 	if (!(rev && *rev))
-		set_revision (contact);
+		set_revision (*contact);
 
-	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
+	vcard = e_vcard_to_string (E_VCARD (*contact), EVC_FORMAT_VCARD_30);
 
 	string_to_dbt (vcard, &vcard_dbt);
 
@@ -197,17 +216,17 @@ do_create(EBookBackendFile  *bf,
 
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
-		if (db_error != 0)
-			g_warning ("db->sync failed with %d", db_error);
-	}
-	else {
-		g_warning (G_STRLOC ": db->put failed with %d", db_error);
-		g_object_unref (contact);
-		contact = NULL;
+		if (db_error != 0) {
+			g_warning ("db->sync failed with %s", db_strerror (db_error));
+		}
+	} else {
+		g_warning (G_STRLOC ": db->put failed with %s", db_strerror (db_error));
+		g_object_unref (*contact);
+		*contact = NULL;
 	}
 
 	g_free (id);
-	return contact;
+	return db_error_to_status (db_error);
 }
 
 static EBookBackendSyncStatus
@@ -217,18 +236,14 @@ e_book_backend_file_create_contact (EBookBackendSync *backend,
 				    const char *vcard,
 				    EContact **contact)
 {
+	EBookBackendSyncStatus status;
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 
-	*contact = do_create (bf, vcard);
-	if (*contact) {
+	status = do_create (bf, vcard, contact);
+	if (status == GNOME_Evolution_Addressbook_Success) {
 		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
-		return GNOME_Evolution_Addressbook_Success;
-	}
-	else {
-		/* XXX need a different call status for this case, i
-                   think */
-		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
+	return status;
 }
 
 static EBookBackendSyncStatus
@@ -254,8 +269,8 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 
 		db_error = db->del (db, NULL, &id_dbt, 0);
 		if (0 != db_error) {
-			g_warning (G_STRLOC ": db->del failed with %d", db_error);
-			rv = GNOME_Evolution_Addressbook_ContactNotFound;
+			g_warning (G_STRLOC ": db->del failed with %s", db_strerror (db_error));
+			rv = db_error_to_status (db_error);
 			continue;
 		}
 
@@ -266,7 +281,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 	if (removed_cards) {
 		db_error = db->sync (db, 0);
 		if (db_error != 0)
-			g_warning (G_STRLOC ": db->sync failed with %d", db_error);
+			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
 	}
 
 	*ids = removed_cards;
@@ -317,7 +332,7 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	/* get the old ecard - the one that's presently in the db */
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 	if (0 != db_error) {
-		g_warning (G_STRLOC ": db->get failed with %d", db_error);
+		g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
 	g_free (vcard_dbt.data);
@@ -332,11 +347,14 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
-		if (db_error != 0)
-			g_warning (G_STRLOC ": db->sync failed with %d", db_error);
-
-		e_book_backend_summary_remove_contact (bf->priv->summary, id);
-		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
+		if (db_error != 0) {
+			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
+		} else {
+			e_book_backend_summary_remove_contact (bf->priv->summary, id);
+			e_book_backend_summary_add_contact (bf->priv->summary, *contact);
+		}
+	} else {
+		g_warning (G_STRLOC ": db->put failed with %s", db_strerror(db_error));
 	}
 	g_free (id);
 	g_free (vcard_with_rev);
@@ -344,7 +362,7 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	if (0 == db_error)
 		return GNOME_Evolution_Addressbook_Success;
 	else
-		return GNOME_Evolution_Addressbook_ContactNotFound;
+		return db_error_to_status (db_error);
 }
 
 static EBookBackendSyncStatus
@@ -372,7 +390,7 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 		*vcard = vcard_dbt.data;
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
-		g_warning (G_STRLOC ": db->get failed with %d", db_error);
+		g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 		*vcard = g_strdup ("");
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
@@ -412,10 +430,10 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 			if (db_error == 0) {
-				contact_list = g_list_append (contact_list, vcard_dbt.data);
+				contact_list = g_list_prepend (contact_list, vcard_dbt.data);
 			} else {
-				g_warning (G_STRLOC ": db->get failed with %d", db_error);
-				status = GNOME_Evolution_Addressbook_OtherError ;
+				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
+				status = db_error_to_status (db_error);
 				break;
 			}
 		}
@@ -428,15 +446,15 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		card_sexp = e_book_backend_sexp_new (search);
 		if (!card_sexp) {
 			/* XXX this needs to be an invalid query error of some sort*/
-			return GNOME_Evolution_Addressbook_ContactNotFound;
+			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 
 		if (db_error != 0) {
-			g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
+			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 			/* XXX this needs to be some CouldNotOpen error */
-			return GNOME_Evolution_Addressbook_ContactNotFound;
+			return db_error_to_status (db_error);
 		}
 
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
@@ -451,7 +469,7 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 
 				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
-					contact_list = g_list_append (contact_list, vcard_dbt.data);
+					contact_list = g_list_prepend (contact_list, vcard_dbt.data);
 				}
 			}
 
@@ -463,13 +481,13 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		if (db_error == DB_NOTFOUND) {
 			status = GNOME_Evolution_Addressbook_Success;
 		} else {
-			g_warning (G_STRLOC ": dbc->c_get failed with %d", db_error);
-			status = GNOME_Evolution_Addressbook_OtherError;
+			g_warning (G_STRLOC ": dbc->c_get failed with %s", db_strerror (db_error));
+			status = db_error_to_status (db_error);
 		}
 
 		db_error = dbc->c_close(dbc);
 		if (db_error != 0) {
-			g_warning (G_STRLOC ": dbc->c_close failed with %d", db_error);
+			g_warning (G_STRLOC ": dbc->c_close failed with %s", db_strerror (db_error));
 		}
 	}
 
@@ -575,7 +593,7 @@ book_view_thread (gpointer data)
 				e_data_book_view_notify_update_prefiltered_vcard (book_view, id, vcard_dbt.data);
 			}
 			else {
-				g_warning (G_STRLOC ": db->get failed with %d", db_error);
+				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 			}
 		}
 
@@ -616,11 +634,12 @@ book_view_thread (gpointer data)
 			}
 
 			dbc->c_close (dbc);
-			if (db_error != DB_NOTFOUND)
-				g_warning ("e_book_backend_file_search: error building list\n");
+			if (db_error && db_error != DB_NOTFOUND)
+				g_warning ("e_book_backend_file_search: error building list: %s",
+					   db_strerror (db_error));
 		}
 		else if (db_error == DB_RUNRECOVERY) {
-			g_warning ("e_book_backend_file_search: error getting the cursor for %s\n",
+			g_warning ("e_book_backend_file_search: error getting the cursor for %s",
 				   bf->priv->filename);
 			abort ();
 		}
@@ -755,7 +774,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
+		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 	} else {
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -774,7 +793,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 				 */
 				contact = create_contact (id_dbt.data, vcard_dbt.data);
 
-#if notyet
+#ifdef notyet
 				g_object_set (card, "last_use", NULL, "use_score", 0.0, NULL);
 #endif
 				vcard_string = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
@@ -807,7 +826,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	if (db_error != DB_NOTFOUND) {
 		g_warning ("e_book_backend_file_changes: error building list\n");
 		*changes_out = NULL;
-		result = GNOME_Evolution_Addressbook_OtherError;
+		result = db_error_to_status (db_error);
 	}
 	else {
 		/* Update the hash and build our changes list */
@@ -942,7 +961,7 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, char *old_version)
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error != 0) {
-			g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
+			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 			return FALSE;
 		}
 
@@ -988,7 +1007,7 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, char *old_version)
 		}
 
 		if (card_failed) {
-			g_warning ("failed to update %d cards\n", card_failed);
+			g_warning ("failed to update %d cards", card_failed);
 			return FALSE;
 		}
 	}
@@ -1072,10 +1091,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	db_error = e_db3_utils_maybe_recover (filename);
 	if (db_error != 0) {
-		g_warning ("db recovery failed with %d", db_error);
+		g_warning ("db recovery failed with %s", db_strerror (db_error));
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		return db_error_to_status (db_error);
 	}
 
 	g_static_mutex_lock(&global_env_lock);
@@ -1085,11 +1104,11 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	} else {
 		db_error = db_env_create (&env, 0);
 		if (db_error != 0) {
-			g_warning ("db_env_create failed with %d", db_error);
+			g_warning ("db_env_create failed with %s", db_strerror (db_error));
 			g_static_mutex_unlock(&global_env_lock);
 			g_free (dirname);
 			g_free (filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			return db_error_to_status (db_error);
 		}
 
 		/* Set the allocation routines to the non-aborting GLib functions */
@@ -1100,11 +1119,11 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		db_error = env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
 		if (db_error != 0) {
 			env->close(env, 0);
-			g_warning ("db_env_open failed with %d", db_error);
+			g_warning ("db_env_open failed with %s", db_strerror (db_error));
 			g_static_mutex_unlock(&global_env_lock);
 			g_free (dirname);
 			g_free (filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			return db_error_to_status (db_error);
 		}
 
 		env->set_errcall (env, file_errcall);
@@ -1118,10 +1137,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	db_error = db_create (&db, env, 0);
 	if (db_error != 0) {
-		g_warning ("db_create failed with %d", db_error);
+		g_warning ("db_create failed with %s", db_strerror (db_error));
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		return db_error_to_status (db_error);
 	}
 
 	db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1130,10 +1149,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		db_error = e_db3_utils_upgrade_format (filename);
 
 		if (db_error != 0) {
-			g_warning ("db format upgrade failed with %d", db_error);
+			g_warning ("db format upgrade failed with %s", db_strerror (db_error));
 			g_free (dirname);
 			g_free (filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			return db_error_to_status (db_error);
 		}
 
 		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1148,7 +1167,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		
         	db_error = db_create (&db, env, 0);
         	if (db_error != 0) {
-                	g_warning ("db_create failed with %d", db_error);
+                	g_warning ("db_create failed with %s", db_strerror (db_error));
                 	g_free (dirname);
                 	g_free (filename);
                 	return GNOME_Evolution_Addressbook_OtherError;
@@ -1161,7 +1180,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			/* the database didn't exist, so we create the
 			   directory then the .db */
 			db->close (db, 0);
-			rv = e_util_mkdir_hier (dirname, 0777);
+			rv = g_mkdir_with_parents (dirname, 0777);
 			if (rv == -1 && errno != EEXIST) {
 				g_warning ("failed to make directory %s: %s", dirname, strerror (errno));
 				g_free (dirname);
@@ -1174,7 +1193,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 			db_error = db_create (&db, env, 0);
 			if (db_error != 0) {
- 				g_warning ("db_create failed with %d", db_error);
+ 				g_warning ("db_create failed with %s", db_strerror (db_error));
 				g_free (dirname);
 				g_free (filename);
 				return GNOME_Evolution_Addressbook_OtherError;
@@ -1183,15 +1202,16 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
 			if (db_error != 0) {
 				db->close (db, 0);
-				g_warning ("db->open (... DB_CREATE ...) failed with %d", db_error);
+				g_warning ("db->open (... DB_CREATE ...) failed with %s", db_strerror (db_error));
 			}
 			else {
 #ifdef CREATE_DEFAULT_VCARD
-				EContact *contact;
+				EContact *contact = NULL;
 
-				contact = do_create(bf, XIMIAN_VCARD);
+				do_create(bf, XIMIAN_VCARD, &contact);
 				/* XXX check errors here */
-				g_object_unref (contact);
+				if (contact)
+					g_object_unref (contact);
 #endif
 
 				writable = TRUE;
@@ -1203,7 +1223,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		bf->priv->file_db = NULL;
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		return db_error_to_status (db_error);
 	}
 
 	if (!e_book_backend_file_maybe_upgrade_db (bf)) {
@@ -1328,6 +1348,22 @@ e_book_backend_file_set_mode (EBookBackend *backend,  GNOME_Evolution_Addressboo
 		e_book_backend_notify_connection_status (backend, TRUE);
 	}
 }
+
+static void 
+e_book_backend_file_sync (EBookBackend *backend)
+{
+	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
+	int db_error;
+	
+	g_return_if_fail (bf != NULL);
+	
+	if (bf->priv->file_db) {
+		db_error = bf->priv->file_db->sync (bf->priv->file_db, 0);
+		if (db_error != 0)
+			g_warning ("db->sync failed with %d", db_error);
+	}
+}
+
 static gboolean
 e_book_backend_file_construct (EBookBackendFile *backend)
 {
@@ -1366,31 +1402,43 @@ e_book_backend_file_dispose (GObject *object)
 
 	bf = E_BOOK_BACKEND_FILE (object);
 
-	if (bf->priv) {
-		if (bf->priv->file_db)
-			bf->priv->file_db->close (bf->priv->file_db, 0);
-
-		g_static_mutex_lock(&global_env_lock);
-		global_env.ref_count--;
-		if (global_env.ref_count == 0) {
-			global_env.env->close(global_env.env, 0);
-			global_env.env = NULL;
-		}
-		g_static_mutex_unlock(&global_env_lock);
-
-		if (bf->priv->summary)
-			g_object_unref(bf->priv->summary);
-		g_free (bf->priv->filename);
-		g_free (bf->priv->dirname);
-		g_free (bf->priv->summary_filename);
-
-		g_free (bf->priv);
-		bf->priv = NULL;
+	if (bf->priv->file_db) {
+		bf->priv->file_db->close (bf->priv->file_db, 0);
+		bf->priv->file_db = NULL;
+	}
+	
+	g_static_mutex_lock(&global_env_lock);
+	global_env.ref_count--;
+	if (global_env.ref_count == 0) {
+		global_env.env->close (global_env.env, 0);
+		global_env.env = NULL;
+	}
+	g_static_mutex_unlock (&global_env_lock);
+	
+	if (bf->priv->summary) {
+		g_object_unref (bf->priv->summary);
+		bf->priv->summary = NULL;
 	}
 
 	G_OBJECT_CLASS (e_book_backend_file_parent_class)->dispose (object);	
 }
 
+static void
+e_book_backend_file_finalize (GObject *object)
+{
+	EBookBackendFile *bf;
+
+	bf = E_BOOK_BACKEND_FILE (object);
+
+	g_free (bf->priv->filename);
+	g_free (bf->priv->dirname);
+	g_free (bf->priv->summary_filename);
+	
+	g_free (bf->priv);
+
+	G_OBJECT_CLASS (e_book_backend_file_parent_class)->finalize (object);
+}
+
 #ifdef G_OS_WIN32
 /* Avoid compiler warning by providing a function with exactly the
  * prototype that db_env_set_func_open() wants for the open method.
@@ -1454,6 +1502,7 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	backend_class->stop_book_view          = e_book_backend_file_stop_book_view;
 	backend_class->cancel_operation        = e_book_backend_file_cancel_operation;
 	backend_class->set_mode                = e_book_backend_file_set_mode;
+	backend_class->sync                = e_book_backend_file_sync;
 	sync_class->remove_sync                = e_book_backend_file_remove;
 	sync_class->create_contact_sync        = e_book_backend_file_create_contact;
 	sync_class->remove_contacts_sync       = e_book_backend_file_remove_contacts;
@@ -1467,6 +1516,7 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	
 
 	object_class->dispose = e_book_backend_file_dispose;
+	object_class->finalize = e_book_backend_file_finalize;
 
 #ifdef G_OS_WIN32
 	/* Use the gstdio wrappers to open, check, rename and unlink
