@@ -23,21 +23,21 @@
 #include <config.h>
 #endif
 
+#include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <ctype.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#include "libedataserver/e-msgport.h"
-#include "libedataserver/e-memory.h"
+#include <libedataserver/e-memory.h>
+#include <libedataserver/e-msgport.h>
 
 #include "camel-block-file.h"
 #include "camel-object.h"
