@@ -24,13 +24,13 @@
 
 #include <string.h>
 
-#include <camel/camel-sasl.h>
-#include <camel/camel-provider.h>
-#include <camel/camel-i18n.h>
+#include <glib/gi18n-lib.h>
+
+#include "camel-provider.h"
+#include "camel-sasl.h"
 
 #include "camel-imap4-store.h"
 
-
 static CamelProviderConfEntry imap4_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "mailcheck", NULL,
 	  N_("Checking for new mail") },
@@ -44,7 +44,7 @@ static CamelProviderConfEntry imap4_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "override_namespace", NULL,
 	  N_("O_verride server-supplied folder namespace"), "0" },
 	{ CAMEL_PROVIDER_CONF_ENTRY, "namespace", "override_namespace",
-	  N_("Namespace") },
+	  N_("Namespace:") },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "general", NULL, N_("Options") },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter", NULL,
