@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#include <string.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreestore.h>
@@ -411,15 +412,24 @@ pixbuf_cell_data_func (GtkTreeViewColumn *column,
 			      NULL);
 	} else {	
 		ESource *source;
-		guint32 color;
 		GdkPixbuf *pixbuf = NULL;
+		const gchar *color_spec;
+		GdkColor color;
 
 		g_assert (E_IS_SOURCE (data));
 		source = E_SOURCE (data);
 		
-		if (e_source_get_color (source, &color)) {
+		color_spec = e_source_peek_color_spec (source);
+		if (color_spec != NULL && gdk_color_parse (color_spec, &color)) {
+			guint32 rgba;
+
 			pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, 16, 16);
-			gdk_pixbuf_fill (pixbuf, color << 8);
+
+			rgba = (((color.red & 0xff00) << 8) |
+				((color.green & 0xff00)) |
+				((color.blue & 0xff00) >> 8)) << 8;
+
+			gdk_pixbuf_fill (pixbuf, rgba);
 		}
 			
 		g_object_set (renderer,
@@ -706,7 +716,7 @@ e_source_selector_class_init (ESourceSelectorClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ESourceSelectorClass, selection_changed),
 			      NULL, NULL,
-			      e_data_server_ui_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
 	signals[PRIMARY_SELECTION_CHANGED] = 
@@ -715,7 +725,7 @@ e_source_selector_class_init (ESourceSelectorClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ESourceSelectorClass, primary_selection_changed),
 			      NULL, NULL,
-			      e_data_server_ui_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 	signals[POPUP_EVENT] =
 		g_signal_new ("popup_event",
@@ -728,6 +738,36 @@ e_source_selector_class_init (ESourceSelectorClass *class)
 			      GDK_TYPE_EVENT|G_SIGNAL_TYPE_STATIC_SCOPE);
 }
 
+static gboolean
+group_search_function   (GtkTreeModel *model,
+			 gint column,
+			 const gchar *key,
+			 GtkTreeIter *iter,
+			 gpointer dummy)
+{
+	void *data;
+	const char *name = NULL;
+	gboolean status = TRUE;
+	
+	gtk_tree_model_get (model, iter, 0, &data, -1);
+
+	if (E_IS_SOURCE_GROUP (data))
+		name = e_source_group_peek_name (E_SOURCE_GROUP (data));
+	else {
+		g_assert (E_IS_SOURCE (data));
+		
+		name = e_source_peek_name (E_SOURCE (data));
+	}
+
+	if (name)
+		status = g_ascii_strncasecmp (name, key, strlen(key)) != 0;
+
+						 
+	g_object_unref (data);
+	
+	return status;
+}
+
 static void
 e_source_selector_init (ESourceSelector *selector)
 {
@@ -738,7 +778,11 @@ e_source_selector_init (ESourceSelector *selector)
 
 	priv = g_new0 (ESourceSelectorPrivate, 1);
 	selector->priv = priv;
-
+	
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (selector), 0);
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (selector), group_search_function, NULL, NULL);
+	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (selector), TRUE);
+	
 	g_signal_connect (G_OBJECT (selector), "button_press_event",
 			  G_CALLBACK (selector_button_press_event), selector);
 
@@ -765,6 +809,7 @@ e_source_selector_init (ESourceSelector *selector)
 
 	cell_renderer = gtk_cell_renderer_text_new ();
 	g_object_set (G_OBJECT (cell_renderer), "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
+	g_object_set (G_OBJECT (cell_renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	gtk_tree_view_column_pack_start (column, cell_renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func (column, cell_renderer, (GtkTreeCellDataFunc) text_cell_data_func, selector, NULL);
 
